;==============================================================================

    include \masm32\include\masm32rt.inc
    include \masm32\include\psapi.inc
    includelib \masm32\lib\psapi.lib

;==============================================================================

    addm64 MACRO dest, src
      mov eax, DWORD PTR src
      mov edx, DWORD PTR src+4
      add DWORD PTR dest, eax
      adc DWORD PTR dest+4, edx
    ENDM

    subm64 MACRO dest, src
      mov eax, DWORD PTR src
      mov edx, DWORD PTR src+4
      sub DWORD PTR dest, eax
      sbb DWORD PTR dest+4, edx
    ENDM

    printf MACRO format:REQ, args:VARARG
      IFNB <args>
        invoke crt_printf, cfm$(format), args
      ELSE
        invoke crt_printf, cfm$(format)
      ENDIF
    ENDM

    dprintf MACRO format:REQ, args:VARARG
      pushfd
      pushad
      IFNB <args>
        invoke crt_printf, cfm$(format), args
      ELSE
        invoke crt_printf, cfm$(format)
      ENDIF
      popad
      popfd
    ENDM

;==============================================================================

    PROCESS_MEMORY_COUNTERS STRUCT
      cb                          DWORD ?
      PageFaultCount              DWORD ?
      PeakWorkingSetSize          DWORD ?
      WorkingSetSize              DWORD ?
      QuotaPeakPagedPoolUsage     DWORD ?
      QuotaPagedPoolUsage         DWORD ?
      QuotaPeakNonPagedPoolUsage  DWORD ?
      QuotaNonPagedPoolUsage      DWORD ?
      PagefileUsage               DWORD ?
      PeakPagefileUsage           DWORD ?
    PROCESS_MEMORY_COUNTERS ENDS

;==============================================================================

    .data?

      sysTime1      dq ?
      sysTime2      dq ?
      kernelTime1   dq ?
      kernelTime2   dq ?
      userTime1     dq ?
      userTime2     dq ?
      dqJunk        dq ?

      hProcess      dd ?
      hThread       dd ?
      hTimer        dd ?

      cpuUsage      dd ?
      lpcallback    dd ?

      pmc           PROCESS_MEMORY_COUNTERS <>

    .code

;==============================================================================

;---------------------------------------------------------------------
; This thread procedure calculates, for the current process, the CPU
; usage percent and the working set size in KB, and passes both to the
; user-supplied callback. Because the loop and callback execute only
; once per timer period, and assuming a timer period of at least
; several hundred milliseconds, their contribution to the CPU usage
; for the process should be negligible.
;---------------------------------------------------------------------

ThreadProc proc lpParameter:DWORD

  @@:

    ;------------------------------------------------------
    ; Get the deltas, in 100-nanosecond intervals, for the
    ; system, kernel, and user times across a timer period
    ; and calculate the process execution time.
    ;------------------------------------------------------

    invoke GetSystemTimeAsFileTime, ADDR sysTime1
    invoke GetProcessTimes, hProcess, ADDR dqJunk, ADDR dqJunk,
                            ADDR kernelTime1, ADDR userTime1

    invoke WaitForSingleObject, hTimer, INFINITE

    invoke GetSystemTimeAsFileTime, ADDR sysTime2
    invoke GetProcessTimes, hProcess, ADDR dqJunk, ADDR dqJunk,
                            ADDR kernelTime2, ADDR userTime2

    subm64 sysTime1, sysTime2         ; sysTime1 = total execution time
    subm64 kernelTime1, kernelTime2   ; kernelTime1 = process kernel time
    subm64 userTime1, userTime2       ; userTime1 = process user time
    addm64 userTime1, kernelTime1     ; userTime1 = process execution time

    ;----------------------------------
    ; Calculate the CPU usage percent.
    ;----------------------------------

    fild userTime1
    fild sysTime1
    fdiv                  ; total execution time / process execution time
    fld8 100.0
    fmul
    fistp cpuUsage        ; percent CPU usage

    ;----------------------------------------------
    ; Get our working set size and scale it to KB.
    ;----------------------------------------------

    invoke GetProcessMemoryInfo, hProcess, ADDR pmc, SIZEOF pmc

    mov eax, pmc.WorkingSetSize
    shr eax, 10

    ;----------------------------------------------------------
    ; Pass the CPU usage and working set size to the callback.
    ;----------------------------------------------------------

    mov edx, lpcallback
    push eax
    push cpuUsage
    call edx

    jmp @B

ThreadProc endp

;==============================================================================

MonitorInit proc updateMs:DWORD, lpCallback:DWORD

    invoke GetCurrentProcess
    mov hProcess, eax

    invoke CreateWaitableTimer, 0, 0, 0
    mov hTimer, eax

    ;---------------------------------------------------------------
    ; Load sysTime1 with the current system time in 100-nanosecond
    ; intervals, advance the value by 3 seconds, and then use its
    ; address as the pDueTime parameter for the SetWaitableTimer
    ; function. This will cause the timer to delay for 3 seconds
    ; before it starts cycling, allowing time for the CPU usage,
    ; etc to settle.
    ;---------------------------------------------------------------

    invoke GetSystemTimeAsFileTime, ADDR sysTime1
    push 0
    push 30000000
    addm64 sysTime1, [esp]
    add esp, 8

    invoke SetWaitableTimer, hTimer, ADDR sysTime1, updateMs, 0, 0, 0

    invoke CreateThread, 0, 0, ThreadProc, 0, CREATE_SUSPENDED, 0
    mov hThread, eax

    ;----------------------------------------------------------------
    ; Increase the priority of the thread to minimize interruptions.
    ; This will set a base priority of 15 for any process priority
    ; other than REALTIME_PRIORITY_CLASS, for which it will set a
    ; base priority of 31 (the maximum).
    ;----------------------------------------------------------------

    invoke SetThreadPriority, hThread, THREAD_PRIORITY_TIME_CRITICAL

    push lpCallback
    pop lpcallback

    invoke ResumeThread, hThread

    ret

MonitorInit endp

;==============================================================================

end
