;==============================================================================
    include \masm32\include\masm32rt.inc
    includelib monitor.lib
    include monitor.inc
;==============================================================================

    printf MACRO format:REQ, args:VARARG
      IFNB <args>
        invoke crt_printf, cfm$(format), args
      ELSE
        invoke crt_printf, cfm$(format)
      ENDIF
    ENDM

;==============================================================================

    .data

      loop_count  dd 10000
      pgup_state  dd 0
      pgdn_state  dd 0
      up_state    dd 0
      down_state  dd 0

    .code

;==============================================================================

CallBack proc cpuUsage:DWORD, workingSet:DWORD

    loc 0, 0
    printf "%d   \t", loop_count
    printf "%d%%\t", cpuUsage
    printf "%dK\n", workingSet

    ret

CallBack endp

;==============================================================================
start:
;==============================================================================

    ;----------------------------------------------------------------
    ; Note that the relationship between the load loop count and the
    ; CPU usage is very non-linear and that the following values may
    ; need to be adjusted to to suit the processor speed.
    ;----------------------------------------------------------------

    UPDATE_MS   equ 2000
    SLEEP_MS    equ 20
    BIG_STEP    equ 1000000
    SMALL_STEP  equ 100000

    invoke MonitorInit, UPDATE_MS, ADDR CallBack

  @@:

    invoke GetKeyState, VK_ESCAPE
    .IF eax & 8000h
        jmp @F
    .ENDIF

    ;------------------------------------------------------------------
    ; Allow the user to control the loop count for the load loop using
    ; PGUP/PGDN (to adjust in steps of BIG_STEP) or UP/DOWN (to adjust
    ; in steps of SMALL_STEP). Lock out the key rollover to limit the
    ; adjustments to one per keystroke. Note that when the loop count
    ; becomes large the user will need to hold the key down until it
    ; is recognized.
    ;------------------------------------------------------------------

    invoke GetKeyState, VK_PGUP
    nops 3
    .IF eax & 8000h
        .IF !(pgup_state & 8000h)
            mov pgup_state, eax
            add loop_count, BIG_STEP
        .ENDIF
    .ELSE
        mov pgup_state, 0
    .ENDIF

    invoke GetKeyState, VK_PGDN
    .IF eax & 8000h
        .IF !(pgdn_state & 8000h)
            mov pgdn_state, eax
            .IF loop_count > BIG_STEP
                sub loop_count, BIG_STEP
            .ENDIF
        .ENDIF
    .ELSE
        mov pgdn_state, 0
    .ENDIF

    invoke GetKeyState, VK_UP
    .IF eax & 8000h
        .IF !(up_state & 8000h)
            mov up_state, eax
            add loop_count, SMALL_STEP
        .ENDIF
    .ELSE
        mov up_state, 0
    .ENDIF

    invoke GetKeyState, VK_DOWN
    .IF eax & 8000h
        .IF !(down_state & 8000h)
            mov down_state, eax
            .IF loop_count > SMALL_STEP
                sub loop_count, SMALL_STEP
            .ENDIF
        .ENDIF
    .ELSE
        mov down_state, 0
    .ENDIF

    mov eax, loop_count
    .WHILE eax
        dec eax
    .ENDW

    invoke Sleep, SLEEP_MS

    jmp @B

  @@:

    inkey "Press any key to exit..."
    exit

;==============================================================================

end start
